<?php
/*
 * Gallery - a web based photo album viewer and editor
 * Copyright (C) 2000-2007 Bharat Mediratta
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 * Perform DAlbum migration
 * @package Migrate
 * @subpackage UserInterface
 * @author Jesse Mullan <jmullan@visi.com>
 * @version $Revision: 15689 $
 */
class ConfirmImportController extends GalleryController {

    /**
     * @see GalleryController::handleRequest
     */
    function handleRequest($form) {
	global $gallery;

	/* Verify that active user is an admin */
	$ret = GalleryCoreApi::assertUserIsSiteAdministrator();
	if ($ret) {
	    return array($ret, null);
	}

	$albumsPath = GalleryUtilities::getRequestVariables('albumsPath');

	/* build framework for error and status arrays */
	$error = array();
	$status = array();
	if (!isset($form['sourceEncoding'])) {
	    $form['sourceEncoding'] = 'UTF-8';
	}

	if (isset($form['action']['cancel'])) {
	    $redirect['view'] = 'core.SiteAdmin';
	    $redirect['subView'] = 'dalbum.SelectGallery';
	} else if (isset($form['action']['import'])) {
	    $templateAdapter =& $gallery->getTemplateAdapter();
	    $templateAdapter->registerTrailerCallback(
		array($this, 'performImport'), array($form, $albumsPath));
	    $results['delegate']['view'] = 'core.ProgressBar';
	}

	if (!empty($redirect)) {
	    $results['redirect'] = $redirect;
	} else if (empty($results['delegate'])) {
	    $results['delegate']['view'] = 'core.SiteAdmin';
	    $results['delegate']['subView'] = 'dalbum.ConfirmImport';
	}
	$results['status'] = $status;
	$results['error'] = $error;

	return array(null, $results);
    }

    /**
     * Actually perform the import
     *
     * @param array $form the form variables
     * @param string $albumsPath the albums path
     * @return object GalleryStatus a status code
     */
    function performImport($form, $albumsPath) {
	global $gallery;
	$storage = $gallery->getStorage();
	$templateAdapter =& $gallery->getTemplateAdapter();

	$platform =& $gallery->getPlatform();
	$slash = $platform->getDirectorySeparator();

	$finishedAlbums = $albumPosition = array();

	$status = array('userImportFailure' => array(),
			'userImportSuccess' => array(),
			'albumImportFailure' => array(),
			'albumImportSuccess' => array(),
			'urlRedirect' => isset($form['urlRedirect']));

	list ($ret, $moduleStatusList) = GalleryCoreApi::fetchPluginStatus('module');
	if ($ret) {
	    return $ret;
	}
	$importComments = !empty($moduleStatusList['comment']['active']);

	if (!empty($form['customFields'])) {
	    list ($ret, $customFieldInterface) =
		GalleryCoreApi::newFactoryInstance('CustomFieldInterface_1_0');
	    if ($ret) {
		return $ret;
	    }
	    if (!isset($customFieldInterface)) {
		return GalleryCoreApi::error(ERROR_MISSING_OBJECT);
	    }
	    $skipCustomItemFields = array();
	    if (!empty($form['skipCustomItemFields'])) {
		foreach (array('summary', 'description') as $field) {
		    if ($form['set'][$field] == 'custom') {
			$skipCustomItemFields[] = $form['customfield'][$field];
		    }
		}
	    }
	}

	list ($ret, $hiddenInterface) = GalleryCoreApi::newFactoryInstance('HiddenInterface_1_0');
	if ($ret) {
	    return $ret;
	}

	/*
	 * owner => core.all (which also gives comment.all)
	 *
	 * canRead => core.view,core.viewResizes
	 * canViewFullImages => core.viewSource
	 * canViewComments =>
	 * comment.view
	 * canAddComments => comment.add
	 * canChangeText => core.edit
	 * canAddTo => core.addDataItem
	 * canWrite => core.edit
	 * canDeleteFrom => core.delete
	 * canCreateSubAlbum => core.addAlbumItem
	 *
	 */

	$permissionTranslation = array('canView' => array('core.view',
							  'core.viewResizes',
				                          'core.viewSource'),
				       );
	if ($importComments) {
	    array_push($permissionTranslation['canView'], 'comment.view');
	}

	/* Get GroupIds for "everybody" and "all users" */
	list ($ret, $everybodyGroupId) =
	    GalleryCoreApi::getPluginParameter('module', 'core', 'id.everybodyGroup');
	if ($ret) {
	    return $ret;
	}
	list ($ret, $allUsersGroupId) =
	    GalleryCoreApi::getPluginParameter('module', 'core', 'id.allUserGroup');
	if ($ret) {
	    return $ret;
	}
	list ($ret, $adminUsersGroupId) =
	    GalleryCoreApi::getPluginParameter('module', 'core', 'id.adminGroup');
	if ($ret) {
	    return $ret;
	}
	list ($ret, $guestUserId) =
	    GalleryCoreApi::getPluginParameter('module', 'core', 'id.anonymousUser');
	if ($ret) {
	    return $ret;
	}

	list ($ret, $module) = GalleryCoreApi::loadPlugin('module', 'dalbum');
	if ($ret) {
	    return $ret;
	}
	$importUsersMessage = $module->translate('Importing Users');
	$importPhotosMessage = $module->translate('Importing Photos');

	$sourceEncoding = $form['sourceEncoding'];

	list ($ret, $markupType) =
	    GalleryCoreApi::getPluginParameter('module', 'core', 'misc.markup');
	if ($ret) {
	    return $ret;
	}

	$itemsCreated = array();

	/* Get list of userids in advance */
	GalleryCoreApi::requireOnce('modules/dalbum/classes/DAlbumDataParser.class');
	list ($ret, $albumList) = DAlbumDataParser::getAlbumList($albumsPath);
	if ($ret) {
	    return $ret;
	}

	list ($ret, $uidMap) = DAlbumDataParser::getUsers($albumsPath);
	if ($ret) {
	    return $ret;
	}

	/* If no users were selected, create an empty array to avoid later php warnings */
	if (!isset($form['migrateUser'])) {
	    $form['migrateUser'] = array();
	}

	/* go through the list of chosen users */
	$totalUsersToImport = count($form['migrateUser']);
	$numberOfUsersImported = 0;
	foreach (array_keys($form['migrateUser']) as $uid) {
	    $gallery->guaranteeTimeLimit(30);

	    /* convert from uid to username */
	    list ($ret, $fields) =
		DAlbumDataParser::getUserFieldsByUid($albumsPath, $uid);
	    if ($ret) {
		return $ret;
	    }
	    $userName = GalleryCoreApi::convertToUtf8($fields['username'], $sourceEncoding);
	    if (empty($totalUsersToImport)) {
		$totalUsersToImport = 1;
	    }
	    $templateAdapter->updateProgressBar(
		$importUsersMessage, $userName, $numberOfUsersImported / $totalUsersToImport);

	    /* Go ahead and create the user instance */
	    list ($ret, $user) = GalleryCoreApi::newFactoryInstance('GalleryEntity', 'GalleryUser');
	    if ($ret) {
		return $ret;
	    }

	    /* Verify that the user was created successfully */
	    if (!isset($user)) {
		return GalleryCoreApi::error(ERROR_MISSING_OBJECT);
	    }

	    /*
	     * Turn the instance into a real boy/girl
	     * This is where the user is actually created, everything
	     * before this was simply building the framework.
	     */
	    $ret = $user->create($userName);
	    if ($ret) {
		/* ERROR_COLLISION is when you try to create something that already exists */
		if (!($ret->getErrorCode() & ERROR_COLLISION)) {
		    return $ret;
		}
		/* Other errors are apparently okay.  Note them and continue */
		$status['userImportFailure'][$userName] = 1;
	    } else {
		/*
		 * Apparently the creation was successful.  Now we can start
		 * setting the various metadata associated with the user
		 */
		$user->setEmail($fields['email']);
		$user->setHashedPassword($fields['password']);
		$fullName = GalleryCoreApi::convertToUtf8(
		    trim($fields['fullname']), $sourceEncoding);
		$fullName = (!empty($fullName)) ? $fullName : $userName;
		$user->setFullName($fullName);

		/*
		 * $fields['canCreateAlbums'] if they have that perm, give them
		 * "core.addAlbumItem" permission on the root album
		 */
		/* Save this user to the database */
		$ret = $user->save();
		if ($ret) {
		    return $ret;
		}
		if (isset($fields['isAdmin']) && $fields['isAdmin']) {
		    /* Add her to the admin group */
		    $ret = GalleryCoreApi::addUserToGroup($user->getId(), $adminUsersGroupId);
		    if ($ret) {
			return $ret;
		    }
		}

		/* Set a positive note for later.  Hooray! */
		$status['userImportSuccess'][$userName] = 1;
		$user = null;
	    }
	    $uid = null;
	    $numberOfUsersImported++;
	}

	$templateAdapter->updateProgressBar($importUsersMessage, '', 1);

	/* Checkpoint to ensure users are saved to the DB in case of a later failure */
	$ret = $storage->checkPoint();
	if ($ret) {
	    return $ret;
	}

	$templateAdapter->resetProgressBarStats();

	/*
	 * Import Any Selected Albums
	 * First initialize the $form['sourceAlbums'] array to avoid php warnings
	 */
	if (!isset($form['sourceAlbums'])) {
	    $form['sourceAlbums'] = array();
	}
	foreach ($form['sourceAlbums'] as $key => $value) {
	    /* See comment for same loop below in ConfirmImportView */
	    unset($form['sourceAlbums'][$key]);
	    $form['sourceAlbums'][urldecode($key)] = $value;
	}

	/*
	 * Check to see if the user selected a destination.  If not,
	 * default to the root album of the new gallery install.
	 */
	if (!isset($form['destinationAlbumID'])) {
	    list ($ret, $rootId) =
		GalleryCoreApi::getPluginParameter('module', 'core', 'id.rootAlbum');
	    if ($ret) {
		return $ret;
	    }
	    $form['destinationAlbumID'] = $rootId;
	}
	/* Convert the form variable into a local temporary variable */
	$destinationAlbumID = $form['destinationAlbumID'];
	/* Go through the list of selected DAlbum albums to import */

	$totalItemsToImport = 0;
	$albumfields = array();
	foreach (array_keys ($form['sourceAlbums']) as $sourceAlbumName) {
	    if (!in_array($sourceAlbumName, array_keys($albumList))) {
		return GalleryCoreApi::error(ERROR_BAD_PARAMETER);
	    }

            list ($ret, $albumfields[$sourceAlbumName]) =
                DAlbumDataParser::loadAlbumFields($albumsPath, $albumList[$sourceAlbumName]);
            if ($ret) {
                return $ret;
            }

	    list ($ret, $photos) = DAlbumDataParser::getPhotos($albumsPath, $sourceAlbumName);
	    if ($ret) {
		return $ret;
	    }
	    $totalItemsToImport += 1 + count($photos);
	}

	/*
	 * Go through the array of albums' metadata one album at a time.
	 * We limit the number of passes through this list to n^2 times to avoid an infinite loop.
	 * This allows us to make sure that we add parent albums first by taking albums out of the
	 * list as they are handled.
	 */
	$i = $numberOfItemsImported = 0;
	while ($albumfields && $i < pow(count($form['sourceAlbums']), 2) && !connection_aborted()) {
	    $gallery->guaranteeTimeLimit(30);
	    $album = array_shift($albumfields);

	    /*
	     * if the album's parent is not in the list of sourcealbums,
	     * load it into the targeted G2 top album as set in $destinationAlbumID
	     */
	    $targetAlbumID = $newAlbumInstanceId = false;
            if (!in_array($album['parentAlbumName'], array_keys($form['sourceAlbums']))) {
                $targetAlbumID = $destinationAlbumID;

	    } else if (in_array($album['parentAlbumName'], array_keys($finishedAlbums))) {
		/* if the album's parent has already been imported, import it into the parent */
		$targetAlbumID = $finishedAlbums[$album['parentAlbumName']];
	    } else {
		$albumfields[$album['path']] = $album;
		$album = null;
	    }
	    if (!$targetAlbumID || !$album) {
		continue;
	    }

	    /* We definitely need notices before each album */
	    $templateAdapter->updateProgressBar($importPhotosMessage,
		    $album['path'], $numberOfItemsImported / $totalItemsToImport);

	    list ($ret, $parent) = GalleryCoreApi::loadEntitiesById($targetAlbumID);
	    if ($ret) {
		return $ret;
	    }
	    list ($ret, $parentPath) = $parent->fetchPath();
	    if ($ret) {
		return $ret;
	    }

	    $desiredname = $platform->legalizePathComponent($album['name']);
	    $k = 0;
	    $invalidName = true;
	    while ($invalidName) {
		list ($ret, $existingAlbumId) =
		    GalleryCoreApi::fetchChildIdByPathComponent($targetAlbumID, $desiredname);
		if ($ret) {
		    if (!$ret->getErrorCode() & ERROR_MISSING_OBJECT) {
			return $ret;
		    } else {
			$invalidName = false;
		    }
		} else {
		    $desiredname = $album['name'] . '_' . $k++;
		}
	    }

	    /* Make sure we have permission to edit the target item */
	    $ret = GalleryCoreApi::assertHasItemPermission($targetAlbumID, 'core.addAlbumItem');
	    if ($ret) {
		return $ret;
	    }

	    /* Try to load targeted parent */
	    list ($ret, $targetAlbumObject) = GalleryCoreApi::loadEntitiesById($targetAlbumID);
	    if ($ret) {
		return $ret;
	    }

	    /* Get a lock on said parent */
	    list ($ret, $importLockIds[]) =
		GalleryCoreApi::acquireReadLock($targetAlbumObject->getId());
	    if ($ret) {
		return $ret;
	    }

	    /* If everything is good so far, we create a new instance to be our new album */
	    if (!empty($desiredname) && $platform->isLegalPathComponent($desiredname)) {
		list ($ret, $newAlbumInstance) =
		    GalleryCoreApi::newFactoryInstance('GalleryEntity', 'GalleryAlbumItem');
		if ($ret) {
		    GalleryCoreApi::releaseLocks($importLockIds);
		    return $ret;
		}
		if (!isset($newAlbumInstance)) {
		    GalleryCoreApi::releaseLocks($importLockIds);
		    return GalleryCoreApi::error(ERROR_MISSING_OBJECT);
		}
		/* this is where the album is actually created */
		$ret = $newAlbumInstance->create($targetAlbumID, $desiredname);
		if ($ret) {
		    GalleryCoreApi::releaseLocks($importLockIds);
		    return $ret;
		}

		if (isset($form['urlRedirect'])) {
		    $ret = $this->createDAlbumToG2Mapping($newAlbumInstance->getId(), $album['path']);
		    if ($ret) {
			GalleryCoreApi::releaseLocks($importLockIds);
			return $ret;
		    }
		}

		$newAlbumInstanceId = $newAlbumInstance->getId();
		$sourceData = array();
		/* load up the album with metadata from the old album */
		$sourceData['title'] =
		    $this->convertHtml($markupType, $album['title'], $sourceEncoding);
		$newAlbumInstance->setTitle($sourceData['title']);
		if (isset($album['comment'])) {
		    $newAlbumInstance->setDescription($this->convertHtml(
				$markupType, $album['comment'], $sourceEncoding));
		} else {
                    $sourceData['summary'] = null;
                }
		$newAlbumInstance->setKeywords('');
		if (isset($album['description'])) {
		    $sourceData['summary'] = $this->convertHtml(
			    $markupType, $album['description'], $sourceEncoding);
		    $newAlbumInstance->setSummary($sourceData['summary']);
		}
		// $newAlbumInstance->settheme
		if (isset($album['createdDate']) && $album['createdDate']) {
		    $newAlbumInstance->setCreationTimestamp($album['createdDate']);
		    $newAlbumInstance->setOriginationTimestamp($album['createdDate']);
		}

		/* $newAlbumInstance->setorderBy */
		/* $newAlbumInstance->setorderDirection */

		/* Save the new album */
		$ret = $newAlbumInstance->save();
		if ($ret) {
		    GalleryCoreApi::releaseLocks($importLockIds);
		    return $ret;
		}
		$numberOfItemsImported++;

		$ret = $this->checkTruncation($newAlbumInstance, $sourceData);
		if ($ret) {
		    GalleryCoreApi::releaseLocks($importLockIds);
		    return $ret;
		}

		/* Create custom fields */
		if (isset($customFieldInterface) && !empty($album['extraFields'])) {
		    $extraFields = $album['extraFields'];
		    if (!empty($extraFields)) {
			foreach ($extraFields as $i => $field) {
			    $extraFields[$i] = $this->convertHtml('html', $field, $sourceEncoding);
			}
			list ($ret) = $customFieldInterface->createCustomFields(
				$extraFields, $newAlbumInstance->getId());
			if ($ret) {
			    return $ret;
			}
		    }
		}

		$itemsCreated[] = $newAlbumInstance->getId();

		/* Set order weight */
		if (isset($albumPosition[$album['path']])) {
		    $weight = $albumPosition[$album['path']];
		} else {
		    list ($ret, $weight) = GalleryCoreApi::fetchExtremeChildWeight(
			    $targetAlbumID, HIGHER_WEIGHT);
		    if ($ret) {
			GalleryCoreApi::releaseLocks($importLockIds);
			return $ret;
		    }
		    $weight += 1000;
		}
		$ret = GalleryCoreApi::setItemOrderWeight($newAlbumInstance->getId(), $weight);
		if ($ret) {
		    GalleryCoreApi::releaseLocks($importLockIds);
		    return $ret;
		}

		/* album permissions */
		$ret = GalleryCoreApi::removeItemPermissions($newAlbumInstance->getId());
		if ($ret) {
		    GalleryCoreApi::releaseLocks($importLockIds);
		    return $ret;
		}
		if ($adminUsersGroupId) {
		    $ret = GalleryCoreApi::addGroupPermission($newAlbumInstance->getId(),
			    $adminUsersGroupId, 'core.all', false);
		    if ($ret) {
			return $ret;
		    }
		}

		if (isset($album['perms'])) {
		    foreach ($album['perms'] as $tmpperm => $tmpusers) {
			foreach ($permissionTranslation[$tmpperm] as $tmpTranslatedPerm) {
			    foreach ($tmpusers as $tmpuser) {
				switch ($tmpuser) {
				case 'valid-user':
				    if ($allUsersGroupId) {
					$ret = GalleryCoreApi::addGroupPermission(
						$newAlbumInstance->getId(),
						$allUsersGroupId,
						$tmpTranslatedPerm,
						false);
					if ($ret) {
					    GalleryCoreApi::releaseLocks($importLockIds);
					    return $ret;
					}
				    }
				    break;
				case 'all':
				    if ($everybodyGroupId) {
					$ret = GalleryCoreApi::addGroupPermission(
						$newAlbumInstance->getId(),
						$everybodyGroupId,
						$tmpTranslatedPerm,
						false);
					if ($ret) {
					    GalleryCoreApi::releaseLocks($importLockIds);
					    return $ret;
					}
				    }
				    break;
				default:
				    $validPermUID = DAlbumDataParser::isValidUid(
					    $albumsPath, $tmpuser);
				    if ($validPermUID) {
					list ($ret, $permsUserFields) =
					    DAlbumDataParser::getUserFieldsByUid(
						    $albumsPath, $tmpuser);
					if ($ret) {
					    GalleryCoreApi::releaseLocks($importLockIds);
					    return $ret;
					}
					$permUsername = $permsUserFields['username'];
					list ($ret, $permUser) =
					    GalleryCoreApi::fetchUserByUsername(
						    GalleryCoreApi::convertToUtf8(
							$permUsername, $sourceEncoding));
					if ($ret &&
						!($ret->getErrorCode() & ERROR_MISSING_OBJECT)) {
					    GalleryCoreApi::releaseLocks($importLockIds);
					    return $ret;
					}
					if ($permUser) {
					    $ret = GalleryCoreApi::addUserPermission(
						    $newAlbumInstance->getId(),
						    $permUser->getId(),
						    $tmpTranslatedPerm,
						    false);
					    if ($ret) {
						GalleryCoreApi::releaseLocks($importLockIds);
						return $ret;
					    }
					}
				    }
				}
			    }
			}
		    }
		}

		if (isset($newAlbumInstanceId)) {
		    $finishedAlbums[$album['path']] = $newAlbumInstanceId;
		    $status['albumImportSuccess'][$album['name']] = $newAlbumInstanceId;
		} else {
		    GalleryCoreApi::releaseLocks($importLockIds);
		    return $ret;
		}
	    }
	    /*
	     * we can let the parent album (and anything else that we might
	     * have tied up) be edited by others now
	     */
	    $ret = GalleryCoreApi::releaseLocks($importLockIds);
	    if ($ret) {
		return $ret;
	    } else {
		$importLockIds = array();
	    }

	    unset($photos);
	    list ($ret, $photos) = DAlbumDataParser::getPhotos($albumsPath, $album['path']);
	    if ($ret) {
		return $ret;
	    }
	    if (!isset($photos)) {
		$photos = array();
	    }

	    list ($ret, $item) = GalleryCoreApi::loadEntitiesById($newAlbumInstanceId);
	    if ($ret) {
		return $ret;
	    }

	    list ($ret, $importLockIds[]) = GalleryCoreApi::acquireReadLock($item->getId());
	    if ($ret) {
		return $ret;
	    }
	    list ($ret, $thumbnailTable) =
		GalleryCoreApi::fetchThumbnailsByItemIds(array($item->getId()));
	    if ($ret) {
		GalleryCoreApi::releaseLocks($importLockIds);
		return $ret;
	    }
	    $albumNeedsThumbnail = empty($thumbnailTable);

	    foreach ($photos as $j => $importAlbumItem) {
		$gallery->guaranteeTimeLimit(30);
		if (isset($importAlbumItem->isAlbumName) && $importAlbumItem->isAlbumName) {
		    list ($ret, $weight) = GalleryCoreApi::fetchExtremeChildWeight(
			    $newAlbumInstanceId, HIGHER_WEIGHT);
		    if ($ret) {
			GalleryCoreApi::releaseLocks($importLockIds);
			return $ret;
		    }
		    $albumPosition[$importAlbumItem->isAlbumName] = $weight + 100 + $j;
		    $importAlbumItem = null;
		} else {
		    $dir = $albumsPath . 'pictures' . $album['path'];
		    $filename = $importAlbumItem['filename'];
		    $filepath = $dir . $filename;

		    $templateAdapter->updateProgressBar(
			    $importPhotosMessage, $album['name'] . ' ' . $filename,
			    $numberOfItemsImported / $totalItemsToImport);

		    list ($base, $extension) = GalleryUtilities::getFileNameComponents($filename);
		    list ($ret, $mimeType) = GalleryCoreApi::convertExtensionToMime($extension);
		    if ($ret) {
			GalleryCoreApi::releaseLocks($importLockIds);
			return $ret;
		    }

		    $convertedComment = $this->convertHtml(
			    $markupType, $importAlbumItem['comment'], $sourceEncoding);

		    $data = array('summary' => '', 'description' => '');
                    $data['title'] = $importAlbumItem['title'];
		    foreach (array_keys($data) as $field) {
			switch ($form['set'][$field]) {
			case 'filename':
			    $data[$field] = $base;
			    break;
			case 'comment':
			    $data[$field] = $convertedComment;
			    break;
			case 'custom':
			    $customField = $form['customfield'][$field];
			    $encodedField = GalleryCoreApi::convertFromUtf8(
				$customField, $form['sourceEncoding']);
			    if (isset($importAlbumItem['extraFields'][$encodedField])) {
				$data[$field] = $this->convertHtml(
					$markupType, $importAlbumItem['extraFields'][$encodedField],
					$sourceEncoding);
			    } else if ($field == 'description' &&
				    isset($form['set']['defaultDescription'])) {
				$data[$field] = $data['summary'];
			    }
			}
		    }

		    list ($ret, $newItem) = GalleryCoreApi::addItemToAlbum(
			    $filepath, $filename, $data['title'], $data['summary'],
			    $data['description'], $mimeType, $item->getId());
		    if ($ret) {
			GalleryCoreApi::releaseLocks($importLockIds);
			return $ret;
		    }
		    $itemsCreated[] = $newItem->getId();
		    $numberOfItemsImported++;

                    $ret = $this->invokeAfterAddHandlers($newItem);
		    if ($ret) {
			GalleryCoreApi::releaseLocks($importLockIds);
			return $ret;
                    }

		    $ret = $this->checkTruncation($newItem, $data);
		    if ($ret) {
			GalleryCoreApi::releaseLocks($importLockIds);
			return $ret;
		    }

		    if (isset($form['urlRedirect'])) {
			$ret = $this->createDAlbumToG2Mapping(
				$newItem->getId(), $album['path'], $importAlbumItem['filename']);
			if ($ret) {
			    GalleryCoreApi::releaseLocks($importLockIds);
			    return $ret;
			}
		    }
		    $itemLockIds = array();
		    list ($ret, $itemLockIds[]) =
			GalleryCoreApi::acquireWriteLock($newItem->getId());
		    if ($ret) {
			GalleryCoreApi::releaseLocks($importLockIds);
			return $ret;
		    }

		    if (isset($importAlbumItem['modifiedDate'])) {
			$newItem->setCreationTimestamp($importAlbumItem['createdDate']);
		    }
		    if (isset($importAlbumItem['createdDate'])) {
			$newItem->setOriginationTimestamp($importAlbumItem['createdDate']);
		    }

		    $ret = $newItem->save();
		    if ($ret) {
			GalleryCoreApi::releaseLocks($importLockIds);
			return $ret;
		    }

		    /* Set custom field values */
		    if (isset($customFieldInterface) && !empty($importAlbumItem['extraFields'])) {
			$extraFields = array();
			foreach ($importAlbumItem['extraFields'] as $key => $value) {
			    if (in_array($key, $skipCustomItemFields)) {
				continue;
			    }
			    $key = $this->convertHtml('html', $key, $sourceEncoding);
			    $extraFields[$key] = $this->convertHtml($markupType, $value,
				    $sourceEncoding);
			}
			if (!empty($extraFields)) {
			    $ret = $customFieldInterface->setCustomFieldValues(
				    $newItem->getId(), $extraFields);
			    if ($ret) {
				return $ret;
			    }
			}
		    }

		    if ($importComments && isset($importAlbumItem['comments']) &&
			    count($importAlbumItem['comments']) > 0) {
			foreach ($importAlbumItem['comments'] as $importComment) {
			    /* Default the comment owner to the guest user. */
			    $commenterId = $guestUserId;

			    /* Add the comment */
			    list ($ret, $comment) = GalleryCoreApi::newFactoryInstance(
				    'GalleryEntity', 'GalleryComment');
			    if ($ret) {
				GalleryCoreApi::releaseLocks($importLockIds);
				return $ret;
			    }

			    if (!isset($comment)) {
				GalleryCoreApi::releaseLocks($importLockIds);
				return GalleryCoreApi::error(ERROR_MISSING_OBJECT);
			    }

			    $ret = $comment->create($newItem->getId());
			    if ($ret) {
				GalleryCoreApi::releaseLocks($importLockIds);
				return $ret;
			    }

			    $comment->setCommenterId($commenterId);
			    $comment->setHost(empty($importComment['IPNumber']) ?
				    'unknown' : $importComment['IPNumber']);
			    if ($commenterId == $guestUserId) {
				/*
				 * We do not have a name field for comments in G2 yet.
				 * We just copy the commenter name from DAlbum into the comment
				 * subject field of G2.
				 */
				$comment->setSubject(
					GalleryCoreApi::convertToUtf8($importComment['name'],
					    $sourceEncoding));
			    } else {
				$comment->setSubject('');
			    }
			    $comment->setComment(
				    ConfirmImportController::convertHtml(
					$markupType, $importComment['commentText'],
					$sourceEncoding));
			    $comment->setDate($importComment['datePosted']);

			    $ret = $comment->save();
			    if ($ret) {
				GalleryCoreApi::releaseLocks($importLockIds);
				return $ret;
			    }
			}
		    }
		    $ret = GalleryCoreApi::releaseLocks($itemLockIds);
		    if ($ret) {
			return $ret;
		    } else {
			$itemLockIds = array();
		    }

		    $status['addFromLocalServer'][] = array('fileName' => $filename,
			    'id' => $newItem->getId());
		}
		/* Checkpoint (keep open files down, save item). */
		$ret = $storage->checkPoint();
		if ($ret) {
		    return $ret;
		}
	    }
	    $templateAdapter->updateProgressBar($importPhotosMessage,
		    $album['path'], $numberOfItemsImported / $totalItemsToImport);

	    /* Checkpoint (keep open files down, save album). */
	    $ret = $storage->checkPoint();
	    if ($ret) {
		return $ret;
	    }

	    $album = null;
	    /* $i keeps track of how many passes we make over the data */
	    $i++;
	}

	/* We just did a lot of permission wrangling; optimize our access lists */
	$ret = GalleryCoreApi::compactAccessLists();
	if ($ret) {
	    return $ret;
	}

	/* Checkpoint to ensure items are saved to the DB in case of a later failure */
	$ret = $storage->checkPoint();
	if ($ret) {
	    return $ret;
	}

	if (isset($importLockIds)) {
	    $ret = GalleryCoreApi::releaseLocks($importLockIds);
	    if ($ret) {
		return $ret;
	    } else {
		$importLockIds = array();
	    }
	}

	/* Checkpoint to ensure highlights are saved to the DB in case of a later failure */
	$ret = $storage->checkPoint();
	if ($ret) {
	    return $ret;
	}

	$templateAdapter->resetProgressBarStats();

	if (!empty($form['generateThumbnails']) && !connection_aborted()) {
	    /*
	     * Get the thumbnails for all of the items we created, in 20 item
	     * increments so that we don't consume too much memory.
	     *
	     * Note: this destroys the $itemsCreated array!
	     */
	    $createThumbsMessage = $module->translate('Creating thumbnails');
	    $itemsCreatedCount = count($itemsCreated);
	    $thumbsCreated = 0;
	    while (!empty($itemsCreated)) {
		$chunk = array_splice($itemsCreated, 0, 20);
		list ($ret, $thumbTable) = GalleryCoreApi::fetchThumbnailsByItemIds($chunk);
		if ($ret) {
		    return $ret;
		}

		foreach ($thumbTable as $thumbnail) {
		    list ($ret, $newThumbnail, $wasRebuilt) =
			GalleryCoreApi::rebuildDerivativeCacheIfNotCurrent($thumbnail->getId());
		    if ($ret) {
			return $ret;
		    }

		    $thumbsCreated++;
		    $templateAdapter->updateProgressBar(
			$createThumbsMessage,
			$module->translate(array('text' => 'Thumbnail %d of %d',
						 'arg1' => $thumbsCreated,
						 'arg2' => $itemsCreatedCount)),
			$thumbsCreated / $itemsCreatedCount);
		}

		/* Checkpoint (keep nr of open files down, save thumbnails). */
		$ret = $storage->checkPoint();
		if ($ret) {
		    return $ret;
		}

		GalleryDataCache::reset();
	    }
	}

	/* Checkpoint to ensure thumbnails are saved to the DB in case of a later failure */
	$ret = $storage->checkPoint();
	if ($ret) {
	    return $ret;
	}

	$session =& $gallery->getSession();

	$redirect['view'] = 'core.SiteAdmin';
	$redirect['subView'] = 'dalbum.ImportResults';
	$redirect['statusId'] = $session->putStatus($status);

	$urlGenerator =& $gallery->getUrlGenerator();
	$templateAdapter->completeProgressBar($urlGenerator->generateUrl($redirect));

	return null;
    }

    /**
     * Convert html according to given markup type
     * @param string $markupType markup type (none, bbcode, html)
     * @param string $item input string
     * @return string
     */
    function convertHtml($markupType, $item, $sourceEncoding='UTF-8') {
	$item = GalleryCoreApi::convertToUtf8($item, $sourceEncoding);
	switch ($markupType) {
	case 'bbcode':
	    $item = ConfirmImportController::convertHtmlToBbcode($item);
	    break;

	case 'html':
	    break;

	case 'none':
	default:
	    $item = strip_tags($item);
	}
	GalleryUtilities::sanitizeInputValues($item, false);
	return $item;
    }

    /**
     * Converts HTML to BBCode
     *
     * @param string $item item to convert
     * @return string
     */
    function convertHtmlToBbcode($item) {
	static $patterns, $match, $replace;
	if (!isset($patterns)) {
	    $patterns = array(
	      '#<br\s*(/?)>#i' => "\n",
	      '#<(/?)(b|strong)>#i' => '[$1b]',
	      '#<(/?)(i|em)>#i' => '[$1i]',
	      '#<(/?)ul>#i' => '[$1list]',
	      '#<li>#i' => '[*]',
	      '#</li>#i' => '',
	      '#<img[^>]*\s+src\s*=\s*["\']{0,1}([^\s"\'>]+)["\']{0,1}[^>]*>#is' => '[img]$1[/img]',
	      '#<a[^>]*\s+href\s*=\s*["\']{0,1}([^\s"\'>]+)["\']{0,1}[^>]*>\s*(.*?)\s*</a>#is' =>
										'[url=$1]$2[/url]',
	    );
	    $match = array_keys($patterns);
	    $replace = array_values($patterns);
	}

	$item = preg_replace($match, $replace, $item);
	return html_entity_decode(strip_tags($item));
    }

    /**
     * Create the DAlbum->G2 map entries so that DAlbum links
     * can be redirected to Gallery2 links.
     * @param int $itemId
     * @param string $g1AlbumName DAlbum album name
     * @param string $g1ItemName (optional) DAlbum item name
     * @return object GalleryStatus a status code
     * @author Joseph Elwell <jelwell@yahoo.com>
     */
    function createDAlbumToG2Mapping($itemId, $albumName, $itemName=null) {
	$ret = GalleryCoreApi::addMapEntry(
	    'DAlbumMigrateMap',
	    array('itemId' => $itemId, 'album' => $albumName, 'albumItem' => $itemName) );
	if ($ret) {
	    return $ret;
	}
	return null;
    }

    function invokeAfterAddHandlers(&$item) {
        if (isset($this->_optionInstances)) {
            $optionInstances = $this->_optionInstances;
        } else {
            GalleryCoreApi::requireOnce('modules/core/ItemAdd.inc');
            list ($ret, $optionInstances) = ItemAddOption::getAllAddOptions();
            if ($ret) {
                return array($ret, null);
            }
        }

        /* Allow ItemAddOptions to process added item(s) */
        $error = array();
        foreach ($optionInstances as $option) {
            list ($ret, $optionErrors, $optionWarnings) =
                $option->handleRequestAfterAdd($form, array($item));
            if ($ret) {
                GalleryCoreApi::releaseLocks($lockId);
                return array($ret, null);
            }
            $error = array_merge($error, $optionErrors);
        }
        if (!empty($error)) {
            return GalleryCoreApi::error(ERROR_UNKNOWN);
        }
        return null;
    }

    /**
     * Print message if any fields were truncated on imported item.
     */
    function checkTruncation(&$item, $sourceData) {
	global $gallery;
	static $titleMessage, $summaryMessage, $setMessage, $truncatedMessage;
	if (!isset($titleMessage)) {
	    list ($ret, $module) = GalleryCoreApi::loadPlugin('module', 'dalbum');
	    if ($ret) {
		return $ret;
	    }
	    $titleMessage = $module->translate('%sTitle%s was truncated for %sitem %d%s.');
	    $summaryMessage = $module->translate('%sSummary%s was truncated for %sitem %d%s.');
	    $setMessage = $module->translate('Set:');
	    $truncatedMessage = $module->translate('Truncated:');
	}

	$data = array($item->getTitle(), $sourceData['title'], $titleMessage,
		      $item->getSummary(), $sourceData['summary'], $summaryMessage);
	while (!empty($data)) {
	    list ($value, $source, $message) = array_splice($data, 0, 3);
	    if ($value != $source) {
		if (!isset($urlGenerator)) {
		    $urlGenerator =& $gallery->getUrlGenerator();
		}
		printf("<div class=\"gbBlock\">$message<br/>" .
		   "<b>$setMessage</b> %s<br/><b>$truncatedMessage</b> %s</div>",
		   '<b>', '</b>',
		   '<a href="' . $urlGenerator->generateUrl(
			array('view' => 'core.ItemAdmin', 'subView' => 'core.ItemEdit',
			      'itemId' => $item->getId(), 'editPlugin' => 'ItemEditItem')) . '">',
		   $item->getId(),
		   '</a>',
		   $value,
		   substr($source, strlen($value)));
	    }
	}

	return null;
    }
}


/**
 * Confirm selections before performing DAlbum migration
 */
class ConfirmImportView extends GalleryView {

    /**
     * @see GalleryView::loadTemplate
     */
    function loadTemplate(&$template, &$form) {
	global $gallery;
	$platform =& $gallery->getPlatform();
	$illegalAlbumNames = $existingAlbums = $titles = $names = array();
	$albumValidOwner = $albumOwnerUserName = $albumOwnerUid = $albumfields = array();
	if (!isset($form['sourceAlbums'])) {
	    $form['sourceAlbums'] = array();
	}
	if (!isset($form['sourceEncoding'])) {
	    $form['sourceEncoding'] = 'ISO-8859-1';
	}
	$albumsPath = GalleryUtilities::getRequestVariables('albumsPath');
	$destinationAlbumID = $form['destinationAlbumID'];
	list ($ret, $uids) = DAlbumDataParser::getUsers($albumsPath);
	if ($ret) {
	    return array($ret, null);
	}
	list ($ret, $albums) = DAlbumDataParser::getAlbumList($albumsPath);
	if ($ret) {
	    return array($ret, null);
	}
	list ($ret, $targetAlbum) = GalleryCoreApi::loadEntitiesById($destinationAlbumID);
	if ($ret) {
	    return array($ret, null);
	}
	foreach ($form['sourceAlbums'] as $key => $value) {
	    /*
	     * We shouldn't really need to encode album names, but we may pass non UTF-8
	     * characters in the form data, so we urlencode these values to avoid mangling
	     * by some browsers.
	     */
	    $form['sourceAlbums'][$key] = urldecode($value);
	}
	foreach ($form['sourceAlbums'] as $sourceAlbumName) {
	    if (!in_array($sourceAlbumName, array_keys($albums))) {
		return array(GalleryCoreApi::error(ERROR_BAD_PARAMETER), null);
	    }
	    $albumfields[$sourceAlbumName] = $albums[$sourceAlbumName];
	}
	$i = 0;
	while ($albumfields) {
	    $existingAlbumId = false;
	    $album = array_shift($albumfields);
	    $albumName = $album['name'];
	    $albumPath = $album['path'];
	    $names[$albumPath] = $albumName;
	    if (!in_array($album['parentAlbumName'], $form['sourceAlbums'])) {
		list ($ret, $existingAlbumId) =
		    GalleryCoreApi::fetchChildIdByPathComponent($destinationAlbumID, $albumName);
	    }
	    if ($ret) {
		if (!$ret->getErrorCode() & ERROR_MISSING_OBJECT) {
		    return array($ret, null);
		}
		$existingAlbums[$albumPath] = false;
	    } else {
		$existingAlbums[$albumPath] = $existingAlbumId;
	    }
	    $titles[$albumPath] =
		GalleryCoreApi::convertToUtf8($album['title'], $form['sourceEncoding']);

	    if (!$platform->isLegalPathComponent($albumName)) {
		$illegalAlbumNames[$albumPath] = $platform->legalizePathComponent($albumName);
	    } else {
		$illegalAlbumNames[$albumPath] = false;
	    }

	    $albumValidOwner[$albumPath] = true;
	    $albumOwnerUserName[$albumPath] = 'admin';
	    $albumOwnerUid[$albumPath] = null;
	    $i++;
	}
	list ($ret, $hiddenInterface) = GalleryCoreApi::newFactoryInstance('HiddenInterface_1_0');
	if ($ret) {
	    return array($ret, null);
	}

	$ConfirmImport = array();
	$ConfirmImport['uids'] = $uids;
	$ConfirmImport['albums'] = $albums;
	$ConfirmImport['albumsPath'] = $albumsPath;
	$ConfirmImport['existingAlbums'] = $existingAlbums;
	$ConfirmImport['illegalAlbumNames'] = $illegalAlbumNames;
	$ConfirmImport['destinationAlbumID'] = $destinationAlbumID;
	$ConfirmImport['albumValidOwner'] = $albumValidOwner;
	$ConfirmImport['albumOwnerUserName'] = $albumOwnerUserName;
	$ConfirmImport['albumOwnerUid'] = $albumOwnerUid;
	$ConfirmImport['targetAlbum'] = (array)$targetAlbum;
	$ConfirmImport['titles'] = $titles;
	$ConfirmImport['names'] = $names;
	$ConfirmImport['canHide'] = isset($hiddenInterface);

	$template->setVariable('ConfirmImport', $ConfirmImport);
	$template->setVariable('controller', 'dalbum.ConfirmImport');
	return array(null, array('body' => 'modules/dalbum/templates/ConfirmImport.tpl'));
    }
}
?>
