<?php
/*
 * Gallery - a web based photo album viewer and editor
 * Copyright (C) 2000-2007 Bharat Mediratta
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 * This controller handles rewriting URLs from DAlbum to the new style Gallery2 locations
 * @package Migrate
 * @subpackage Redirect
 * @author Joseph Elwell <jelwell@yahoo.com>
 * @version $Revision: 15513 $
 */
class RedirectController extends GalleryController {

    /**
     * @see GalleryController::shouldSaveSession
     */
    function shouldSaveSession() {
	return false;
    }

    /**
     * @see GalleryController::omitAuthTokenCheck
     */
    function omitAuthTokenCheck() {
	return true;
    }

    /**
    * @see GalleryController::handleRequest
    */
    function handleRequest($form) {
	global $gallery;
	GalleryCoreApi::requireOnce('modules/dalbum/classes/DAlbumMigrateHelper.class');

	$path = GalleryUtilities::getRequestVariables('path');
        switch ($path) {
        case 'showimg.php':
	    $file = $_GET['file'];
            list($albumName, $albumItem) = RedirectController::parseFile($file);
            $view = 'core.ShowItem';
            break;
        case 'photo.php':
	    $file = $_GET['file'];
            list($albumName, $albumItem) = RedirectController::parseFile($file);
            $view = 'core.DownloadItem';
            break;
        case 'index.php':
        default:
	    $albumName = $_GET['folder'];
            $view = 'core.ShowItem';
            break;
        }

        if (isset($albumName)) {
	    list ($ret, $id) = DAlbumMigrateHelper::fetchMapping($albumName,
                $albumItem);
	    if ($ret) {
	        return array($ret, null);
            }
        }

	if (isset($id)) {
            $results = array('redirect' => array('view' => $view,
                'itemId' => $id), 'status' => array(), 'error' => array());
        } else {
            if (isset($albumName) && $albumName != '/') {
                return array(GalleryCoreApi::error(ERROR_MISSING_OBJECT), null);
            } else {
	        $results = array('redirect' => array(),
                    'status' => array(), 'error' => array());
            }
	}

	$scriptName = GalleryUtilities::getServerVar('PHP_SELF');
	if (empty($scriptName)) {
	    /* PHP as CGI needs to use PHP_SELF; fallback to SCRIPT_NAME just in case */
	    $scriptName = GalleryUtilities::getServerVar('SCRIPT_NAME');
	}

	if (strpos($scriptName, GALLERY_MAIN_PHP) !== false) {
	    /*
	     * $scriptName may not be the right path.. see http://bugs.php.net/bug.php?id=31843
	     * If not, omit 'href' in redirect params which means we hope REQUEST_URI has been
	     * rewritten to the correct path (that is the case on php 5.0.3 cgi at least).
	     */
	    $urlGenerator =& $gallery->getUrlGenerator();
	    $results['redirect']['href'] = $urlGenerator->makeUrl($scriptName);
	}

	list ($full, $page) = GalleryUtilities::getRequestVariablesNoPrefix('full', 'page');
	if (!empty($full)) {
	    $results['redirect']['imageViewsIndex'] = 1;
	}
	if (!empty($page)) {
	    $results['redirect']['page'] = $page;
	}

	return array(null, $results);
    }

    function parseFile($file) {
        ereg('(.*/)([^/]*)', $file, $reg);
        $albumName = $reg[1];
        $albumItem = $reg[2];
        return array($albumName, $albumItem);
    }
}

/**
 * View returns .htaccess file
 */
class RedirectView extends GalleryView {
    /**
     * @see GalleryView::isImmediate
     */
    function isImmediate() {
	return true;
    }

    /**
     * @see GalleryView::renderImmediate
     */
    function renderImmediate($status, $error) {
	$ret = GalleryCoreApi::assertUserIsSiteAdministrator();
	if ($ret) {
	    return $ret;
	}

	header('Content-type: text/plain');

	GalleryCoreApi::requireOnce('modules/core/classes/GalleryTemplate.class');
	$template = new GalleryTemplate(dirname(dirname(dirname(__FILE__))));
	$template->setVariable('l10Domain', 'modules_migrate');
	$template->display('gallery:modules/dalbum/templates/Htaccess.tpl');

	return null;
    }
}
?>
