<?php
/*
 * Gallery - a web based photo album viewer and editor
 * Copyright (C) 2000-2007 Bharat Mediratta
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 * DAlbum Module
 *
 * This module provides support for Migrating from DAlbum
 *
 * @package DAlbum
 * @author Chad Parry <spam@chad.parry.org>
 * @author Bharat Mediratta <bharat@menalto.com>
 * @version $Revision: 16034 $
 */
class DAlbumModule extends GalleryModule /* and GalleryEventListener */ {
    function DAlbumModule() {
	global $gallery;

	$this->setId('dalbum');
	$this->setName($gallery->i18n('DAlbum'));
	$this->setDescription($gallery->i18n('Migrate your DAlbum to Gallery 2'));
	$this->setVersion('0.1.0');
	$this->setGroup('import', $gallery->i18n('Import'));
	$this->setCallbacks('registerEventListeners|getSiteAdminViews');
	$this->setRequiredCoreApi(array(7, 4));
	$this->setRequiredModuleApi(array(3, 0));
    }

    /**
     * @see GalleryModule::install
     *
     * We pushed all this code into its own file since we need it very rarely so it doesn't
     * make sense to load it every single time.
     */
    function upgrade($currentVersion, $statusMonitor) {
	global $gallery;
	$storage =& $gallery->getStorage();

	/* We don't care about anything older than 1.0.4 */
	if (version_compare($currentVersion, '1.0.4', '<=')) {
	    $currentVersion = '1.0.4';
	}

	switch($currentVersion) {
	case '1.0.4':
	    /* Added index */
	    $ret = $storage->configureStore($this->getId(), array('DAlbumMigrateMap:1.0'));
	    if ($ret) {
		return $ret;
	    }

	case 'end of upgrade path':
	    /*
	     * Leave this bogus case at the end of the legitimate case statements so that we
	     * always properly terminate our upgrade path with a break.
	     */
	    break;
	}

	return null;
    }

    /**
     * @see GalleryModule::registerEventListeners
     */
    function registerEventListeners() {
	GalleryCoreApi::registerEventListener('GalleryEntity::delete', new DAlbumModule(), true);
    }

    /**
     * @see GalleryEventListener::handleEvent
     */
    function handleEvent($event) {
	$item = $event->getEntity();
	if (GalleryUtilities::isA($item, 'GalleryItem')) {
	    $ret = GalleryCoreApi::removeMapEntry('DAlbumMigrateMap',
						  array('itemId' => $item->getId()));
	    if ($ret) {
		return array($ret, null);
	    }
	}
	return array(null, null);
    }

    /**
     * @see GalleryModule::getSiteAdminViews
     */
    function getSiteAdminViews() {
	global $gallery;
	return array(null,
		     array(array('name' => $this->translate('DAlbum'),
				 'view' => 'dalbum.SelectGallery')));
    }

    /**
     * @see GalleryModule::getRewriteRules
     */
    function getRewriteRules() {
	global $gallery;
	$urlGenerator =& $gallery->getUrlGenerator();
        $baseDir = $urlGenerator->generateUrl(
            array('href' => ''),
            array('forceServerRelativeUrl' => 1, 'forceDirect' => 1));
        $basePath = $urlGenerator->generateUrl(
            array('forceServerRelativeUrl' => 1, 'forceDirect' => 1));
	$url = $urlGenerator->generateUrl(array('view' => 'core.SiteAdmin',
						'subView' => 'dalbum.SelectGallery'));
	return array(
	    array('comment' => $this->translate('DAlbum-&gt;G2 Mapping'),
		  'parser' => 'preGallery',
		  'conditions' => array(
                      array('test' => 'REQUEST_URI', 'pattern' => '^' . $baseDir . '(index\.php|showimg\.php|photo\.php)$', flags => array('OR')),
                      array('test' => 'QUERY_STRING', 'pattern' => '(^|&)(folder|file)='),
                      array('test' => 'QUERY_STRING', 'pattern' => '!(^|&)g2_controller=')),
		  'queryString' => array('controller' => 'dalbum.Redirect', 'path' => '%1'),
                  'flags' => array('QSA'),
		  'help' =>$this->translate(
		      array('text' => 'DAlbum URL redirect. Use only if G2 is installed where DAlbum used '
			  . 'to be and you have run a DAlbum import with the URL redirect option '
			  . 'selected. Otherwise follow instructions in %sMigrate Site Admin%s.',
			    'arg1' => '<a href="' . $url . '">', 'arg2' => '</a>'))
	    )
	);
    }
}
?>
